import itertools
import random

from scoremipsum.score import generate_score_anyball, generate_score_hockey, generate_score_football, \
    generate_score_baseball, generate_score_basketball
from scoremipsum.util.scheduler import grouper


def generate_games_from_schedule(schedule, gametype=None):
    """
    given a schedule and game type
    return a list of game_results with scores

    - needs to implement new schedule data (home / away)

    :param schedule:
    :param gametype:
    :return:
    """
    game_results = []

    print('\ngenerating game results for: ', gametype)
    for game in schedule:
        # print('game: ', game)
        # result_score = generate_score_anyball()

        # *NOTE*
        # match command available at python version 3.10
        # -----------------------------------------------------------------------------
        # match gametype:
        #     case 'hockey':
        #         result_score = generate_score_hockey()
        #     case 'football':
        #         result_score = generate_score_football()
        #     case 'anyball':
        #         result_score = generate_score_anyball()
        #     case _:
        #         result_score = generate_score_anyball()

        #   using anyball as default
        if gametype is None:
            score = generate_score_anyball()
        elif gametype == 'baseball':
            score = generate_score_baseball()
        elif gametype == 'basketball':
            score = generate_score_basketball()
        elif gametype == 'hockey':
            score = generate_score_hockey()
        elif gametype == 'football':
            score = generate_score_football()
        elif gametype == 'anyball':
            score = generate_score_anyball()
        else:
            score = generate_score_anyball()

        # print('-- result_score:', result_score)
        # for team_score in result_score:
        #     print(f'-- {team_score = }:', )

        game_results.append(list(zip(game, score)))

    return game_results


def generate_schedule_all_pairs(teamlist):
    """

    :param teamlist:
    :return:
    """
    # if not teamlist:
    #     return something-hardcoded
    # generate all non-repeating pairs - placeholder algorithm
    pairs = list(itertools.combinations(teamlist, 2))

    # randomly shuffle these pairs
    #   also, change test to account for shuffling
    # random.shuffle(pairs)
    return pairs


def generate_schedule_single_pairs(teamlist):
    """

    :param teamlist:
    :return:
    """
    # if not teamlist:
    #     return something-hardcoded
    # generate first non-repeating pairs - placeholder algorithm
    tmp = list(teamlist)
    randoms = [tmp.pop(random.randrange(len(tmp))) for _ in range(len(teamlist))]
    # print('result: ', randoms)
    pairs = list(grouper(randoms, 2))
    # print('pairs: ', pairs)
    return pairs



#
#   SCOREM
#
"""
Scorem
----------

Scorem functions for the `scoremipsum` module.
"""
from scoremipsum.schedule import generate_schedule_single_pairs, generate_games_from_schedule
from scoremipsum.util.conversion import convert_game_result_to_json
from scoremipsum.util.support import get_supported_sports
from scoremipsum.util.team import get_default_teamlist_from_gametype


def game(gametype=None):
    # print(f"game({gametype=}) not yet implemented !")
    print(f"game({gametype=}) ... ")

    # DE5: game(gametype=foo) gives incorrect teams in results.
    #   error here, wrong teamlist for schedule when gametype is not None
    # teamlist = data.TEAMS_DEFAULT
    teamlist = get_default_teamlist_from_gametype(gametype)

    schedule = generate_schedule_single_pairs(teamlist)
    game_generation_results = generate_games_from_schedule(schedule, gametype=gametype)
    game_results_json = convert_game_result_to_json(game_generation_results, gametype=gametype)
    return game_results_json


def commands():

    # DYNAMIC determination, now returns convert_game_result_to_json() method - mess with this later
    # method_list = [func for func in dir(scoremipsum.scoremipsum) if
    #                callable(getattr(scoremipsum.scoremipsum, func)) and not func.startswith(
    #                    "_") and not func.startswith("get_")]
    # for now, maintain the command list manually
    method_list = ['commands', 'game', 'help', 'sports', 'sportsball']
    return method_list


def help():
    print("== help()")
    print("-" * 9)
    print("== Use the following commands to get started quickly:")
    print("== ")
    print("==       ops.sportsball()")
    print("==           Displays 'Sportsball' as a sanity check")
    print("== ")
    print("==       ops.help()")
    print("==           Displays this content")
    print("== ")
    print("==       ops.commands()")
    print("==           Displays a list of supported commands")
    print("==           e.g. commands=['commands', 'game', 'help', 'sports', 'sportsball']")
    print("== ")
    print("==       ops.sports()")
    print("==           Displays a list of supported sports")
    print("==           e.g. sports=['anyball', 'baseball', 'basketball', 'football', 'hockey']")
    print("== ")
    print("==       ops.game()")
    print("==           Generates game scores for default sport, using default generic team list")
    print("==           Returns JSON object")
    print("==           Syntax:")
    print("==               results = ops.game()")
    print("== ")
    print("==       ops.game(gametype='<gametype>')")
    print("==           Generates game scores for selected sport, using default team list from that sport")
    print("==           Returns JSON object")
    print("==           Syntax:")
    print("==               results = ops.game(gametype='hockey')")
    print("== ")

    print("-" * 80)


def sportsball():
    print("== sportsball !")
    print("-" * 80)


def sports():
    sports_list = get_supported_sports()
    return sports_list



#
#   SCOREMIPSUM
#
"""
init
----------

init for the `scoremipsum` module.
"""

__version__ = "0.0.6"


def game():
    return None


def generation():
    return None


def score():
    return None



"""
   scoremipsum demo test
"""
import sys

import ops


# import scoremipsum

# from scoremipsum import scoremipsum


def main():
    """
    scoremipsum demo
    """
    print("="*80)
    print('(scoremipsum demo) :: demo ::')
    print("-"*80)

    #   display the supported sports list
    #
    ops.sportsball()

    # scoremipsum.help()

    # commands = scoremipsum.commands()
    # print(f"== {commands=}")
    print("-"*80)

    # sports = scoremipsum.sports()
    # print(f"== {sports=}")
    print("-"*80)

    print('(scoremipsum demo) :: end ::')
    print("="*80)
    return 0


# ----------------------------------------
if __name__ == '__main__':
    main()
    sys.exit(0)



#
#   SCOREM
#
"""
team
----------

team utils for the `scoremipsum` module.
"""
from scoremipsum import data


def get_default_teamlist_from_gametype(gametype=None):

    if not gametype:
        teamlist = data.TEAMS_DEFAULT
    elif gametype == "baseball":
        teamlist = data.TEAMS_MLB_DEFAULT
    elif gametype == "basketball":
        teamlist = data.TEAMS_NBA_DEFAULT
    elif gametype == "football":
        teamlist = data.TEAMS_NFL_DEFAULT
    elif gametype == "hockey":
        teamlist = data.TEAMS_NHL_DEFAULT
    else:
        # anyball, or any unhandled gametype
        teamlist = data.TEAMS_DEFAULT

    return teamlist


def get_team_data(team=None):
    """

    :param team:    identifier for specific team.
                    must also differentiate for specific sport.
    :return:
    """
    team_data = {'Offense': 2, 'Defense': 2, 'Special': 2}
    return team_data



#!/usr/bin/env python
#
#   SCOREM
#
"""
test_scorem
----------

Tests for the `scoremipsum` module.
"""
import json

import pytest

import scoremipsum
from scoremipsum import data
from scoremipsum.data import TEAMS_NFL_AFC_EAST
from scoremipsum.ops import sports
from scoremipsum.schedule import generate_games_from_schedule, generate_schedule_single_pairs
from scoremipsum.score import (generate_score_anyball, generate_score_hockey,
                               generate_score_football, generate_score_baseball, generate_score_basketball)
from scoremipsum.util.conversion import convert_game_result_to_json
from scoremipsum.util.support import is_valid_json
from scoremipsum.util.team import get_team_data


@pytest.fixture()
def teamlist_nfl_afc_east():
    return TEAMS_NFL_AFC_EAST


def test_data_get_teamlist_nfl_afc_east():
    assert data.TEAMS_NFL_AFC_EAST == ['Patriots', 'Bills', 'Dolphins', 'Jets']


def test_data_get_via_fixture_teamlist_nfl_afc_east(teamlist_nfl_afc_east):
    assert data.TEAMS_NFL_AFC_EAST == teamlist_nfl_afc_east


def test_game_get_team_default_values():
    team_data = get_team_data()
    assert team_data == {'Offense': 2, 'Defense': 2, 'Special': 2}


def test_game_get_teamlist_default():
    assert scoremipsum.data.TEAMS_DEFAULT == ['Advancers', 'Battlers', 'Clashers', 'Destroyers', 'Engineers',
                                              'Fighters', 'Guardians', 'Harriers']


def test_score_generate_score_anyball():
    """
        simulated result_score for single game of anyball (imaginary)

    :return:
    """
    # return 2 ints, range 0-99
    game_score = generate_score_anyball()
    assert 100 > game_score[0] >= 0
    assert 100 > game_score[1] >= 0
    print(f"\nresult_score = {game_score}")


def test_score_generate_score_baseball():
    """
        simulated result_score for single game of anyball (imaginary)

    :return:
    """
    # return 2 ints, range 0-99
    game_score = generate_score_baseball()
    assert 100 > game_score[0] >= 0
    assert 100 > game_score[1] >= 0
    print(f"\nresult_score = {game_score}")


def test_score_generate_score_basketball():
    """
        simulated result_score for single game of anyball (imaginary)

    :return:
    """
    # return 2 ints, range 0-149
    game_score = generate_score_basketball()
    assert 150 > game_score[0] >= 0
    assert 150 > game_score[1] >= 0
    print(f"\nresult_score = {game_score}")


def test_score_generate_score_football():
    """
        simulated result_score for single game of football
        nfl record for single team result_score is 73
        nfl record for both teams combined result_score is 113
    :return:
    """
    # return 2 ints, range 0-74, total < 120
    # this will be weighted for realism and tests adjusted
    game_score = generate_score_football()
    assert 75 > game_score[0] >= 0
    assert 75 > game_score[1] >= 0
    assert 120 > (game_score[0] + game_score[1]) >= 0
    print(f"\nresult_score = {game_score}")


def test_score_generate_score_hockey():
    """
        simulated result_score for single game of hockey
        nhl record for single team result_score is 16
        nhl record for both teams combined result_score is 21
    :return:
    """
    # return 2 ints, range 0-16, total < 22
    # this will be weighted for realism and tests adjusted
    game_score = generate_score_hockey()
    assert 17 > game_score[0] >= 0
    assert 17 > game_score[1] >= 0
    assert 22 > (game_score[0] + game_score[1]) >= 0
    print(f"\nresult_score = {game_score}")


# test invalid until delivery of US111: SCOREM - Specify and Enforce "Away - Home" in Schedule
@pytest.mark.skip(reason='US111')
def test_generate_schedule_single_pairs():
    schedule_set = ('always_team_AWAY', 'always_team_HOME')
    schedule = generate_schedule_single_pairs(schedule_set)
    assert schedule[0][0] == 'always_team_AWAY'
    assert schedule[0][1] == 'always_team_HOME'


def test_generate_games_from_schedule():
    schedule_set = ('always_team_AWAY', 'always_team_HOME')
    schedule = scoremipsum.schedule.generate_schedule_single_pairs(schedule_set)
    game_results = \
        generate_games_from_schedule(schedule, gametype='anyball')
    assert game_results is not None


def test_get_supported_sports_from_root():
    sports_list = sports()
    assert sports_list == ['anyball', 'baseball', 'basketball', 'football', 'hockey']


def test_get_supported_sports_from_util():
    sports_list = scoremipsum.util.support.get_supported_sports()
    assert sports_list == ['anyball', 'baseball', 'basketball', 'football', 'hockey']


def test_is_supported_anyball():
    assert scoremipsum.util.support.check_support_anyball() is True


def test_is_supported_baseball():
    assert scoremipsum.util.support.check_support_baseball() is True


def test_is_supported_basketball():
    assert scoremipsum.util.support.check_support_basketball() is True


def test_is_supported_football():
    assert scoremipsum.util.support.check_support_football() is True


def test_is_supported_hockey():
    assert scoremipsum.util.support.check_support_hockey() is True


def test_result_single_anyball():
    # schedule_set = ('Anyball_Away', 'Anyball_Home')
    schedule_set = ('Anyball_Team_AA', 'Anyball_Team_BB')
    schedule = scoremipsum.schedule.generate_schedule_single_pairs(schedule_set)
    game_generation_results = \
        generate_games_from_schedule(schedule, gametype='anyball')
    assert len(schedule_set) // 2 == len(game_generation_results)

    # verify US96: Results reduce ties.  Temporary until ties are permitted.
    # assert game_generation_results[0][0][1] != game_generation_results[0][1][1]

    game_results_json = convert_game_result_to_json(game_generation_results, gametype='anyball')
    print(f"{game_results_json=}")

    is_good_json = is_valid_json(game_results_json)
    assert is_good_json is True
    # NOT GOOD ENOUGH FOR JSON CONTENT CHECKS THOUGH!

    gametype = json.loads(game_results_json)[0]["gametype"]
    assert gametype == "anyball"


def test_result_single_baseball():
    # schedule_set = ('Baseball_Away', 'Baseball_Home')
    schedule_set = ('Baseball_Team_AA', 'Baseball_Team_BB')
    schedule = scoremipsum.schedule.generate_schedule_single_pairs(schedule_set)
    game_generation_results = \
        generate_games_from_schedule(schedule, gametype='baseball')
    assert len(schedule_set) // 2 == len(game_generation_results)

    # verify US96: Results reduce ties.  Temporary until ties are permitted.
    assert game_generation_results[0][0][1] != game_generation_results[0][1][1]

    game_results_json = convert_game_result_to_json(game_generation_results, gametype='baseball')
    print(f"{game_results_json=}")

    is_good_json = is_valid_json(game_results_json)
    assert is_good_json is True
    # NOT GOOD ENOUGH FOR JSON CONTENT CHECKS THOUGH!

    gametype = json.loads(game_results_json)[0]["gametype"]
    assert gametype == "baseball"


def test_result_single_basketball():
    # schedule_set = ('Basketball_Away', 'Basketball_Home')
    schedule_set = ('Basketball_Team_AA', 'Basketball_Team_BB')
    schedule = scoremipsum.schedule.generate_schedule_single_pairs(schedule_set)
    game_generation_results = \
        generate_games_from_schedule(schedule, gametype='basketball')
    assert len(schedule_set) // 2 == len(game_generation_results)

    # verify US96: Results reduce ties.  Temporary until ties are permitted.
    assert game_generation_results[0][0][1] != game_generation_results[0][1][1]

    game_results_json = convert_game_result_to_json(game_generation_results, gametype='basketball')
    print(f"{game_results_json=}")

    is_good_json = is_valid_json(game_results_json)
    assert is_good_json is True
    # NOT GOOD ENOUGH FOR JSON CONTENT CHECKS THOUGH!

    gametype = json.loads(game_results_json)[0]["gametype"]
    assert gametype == "basketball"


def test_result_single_football():
    # schedule_set = ('Football_Away', 'Football_Home')
    schedule_set = ('Football_Team_AA', 'Football_Team_BB')
    schedule = scoremipsum.schedule.generate_schedule_single_pairs(schedule_set)
    game_generation_results = \
        generate_games_from_schedule(schedule, gametype='football')
    assert len(schedule_set) // 2 == len(game_generation_results)
    # print(f"{game_generation_results=}")

    # verify US96: Results reduce ties.  Temporary until ties are permitted.
    assert game_generation_results[0][0][1] != game_generation_results[0][1][1]

    game_results_json = convert_game_result_to_json(game_generation_results, gametype='football')
    print(f"{game_results_json=}")

    is_good_json = is_valid_json(game_results_json)
    assert is_good_json is True
    # NOT GOOD ENOUGH FOR JSON CONTENT CHECKS THOUGH!

    gametype = json.loads(game_results_json)[0]["gametype"]
    assert gametype == "football"


def test_result_single_hockey():
    # schedule_set = ('Hockey_Away', 'Hockey_Home')
    schedule_set = ('Hockey_Team_AA', 'Hockey_Team_BB')
    schedule = scoremipsum.schedule.generate_schedule_single_pairs(schedule_set)
    game_generation_results = \
        generate_games_from_schedule(schedule, gametype='hockey')
    assert len(schedule_set) // 2 == len(game_generation_results)
    # print(f"{game_generation_results=}")

    # verify US96: Results reduce ties.  Temporary until ties are permitted.
    assert game_generation_results[0][0][1] != game_generation_results[0][1][1]

    game_results_json = convert_game_result_to_json(game_generation_results, gametype='hockey')
    print(f"{game_results_json=}")

    gametype = json.loads(game_results_json)[0]["gametype"]
    assert gametype == "hockey"


def test_result_multiple_anyball():
    schedule_set = ('AA', 'BB', 'CC', 'DD')
    schedule = scoremipsum.schedule.generate_schedule_single_pairs(schedule_set)
    game_generation_results = \
        generate_games_from_schedule(schedule, gametype='anyball')
    assert len(schedule_set) // 2 == len(game_generation_results)
    # print(f"{game_generation_results=}")

    multi_game_results_json = convert_game_result_to_json(game_generation_results, gametype='anyball')
    print(f"{multi_game_results_json=}")

    gametype = json.loads(multi_game_results_json)[0]["gametype"]
    assert gametype == "anyball"


def test_result_multiple_baseball():
    schedule_set = data.TEAMS_MLB_AL_EAST
    schedule = scoremipsum.schedule.generate_schedule_single_pairs(schedule_set)
    game_generation_results = \
        generate_games_from_schedule(schedule, gametype='baseball')
    assert len(schedule_set) // 2 == len(game_generation_results)
    # print(f"{game_generation_results=}")

    multi_game_results_json = convert_game_result_to_json(game_generation_results, gametype='baseball')
    print(f"{multi_game_results_json=}")

    gametype = json.loads(multi_game_results_json)[0]["gametype"]
    assert gametype == "baseball"


def test_result_multiple_basketball():
    schedule_set = data.TEAMS_NBA_EASTERN_CENTRAL
    schedule = scoremipsum.schedule.generate_schedule_single_pairs(schedule_set)
    game_generation_results = \
        generate_games_from_schedule(schedule, gametype='basketball')
    assert len(schedule_set) // 2 == len(game_generation_results)
    # print(f"{game_generation_results=}")

    multi_game_results_json = convert_game_result_to_json(game_generation_results, gametype='basketball')
    print(f"{multi_game_results_json=}")

    gametype = json.loads(multi_game_results_json)[0]["gametype"]
    assert gametype == "basketball"


def test_result_multiple_football():
    schedule_set = data.TEAMS_NFL_AFC_EAST
    schedule = scoremipsum.schedule.generate_schedule_single_pairs(schedule_set)
    game_generation_results = \
        generate_games_from_schedule(schedule, gametype='football')
    assert len(schedule_set) // 2 == len(game_generation_results)
    # print(f"{game_generation_results=}")

    multi_game_results_json = convert_game_result_to_json(game_generation_results, gametype='football')
    print(f"{multi_game_results_json=}")

    gametype = json.loads(multi_game_results_json)[0]["gametype"]
    assert gametype == "football"


def test_result_multiple_hockey():
    schedule_set = data.TEAMS_NHL_EASTERN_ATLANTIC
    schedule = scoremipsum.schedule.generate_schedule_single_pairs(schedule_set)
    game_generation_results = \
        generate_games_from_schedule(schedule, gametype='hockey')
    assert len(schedule_set) // 2 == len(game_generation_results)
    # print(f"{game_generation_results=}")

    multi_game_results_json = convert_game_result_to_json(game_generation_results, gametype='hockey')
    print(f"{multi_game_results_json=}")

    gametype = json.loads(multi_game_results_json)[0]["gametype"]
    assert gametype == "hockey"


def test_schedule_all_pairs():
    schedule_set = ('AA', 'BB', 'CC', 'DD')
    schedule = scoremipsum.schedule.generate_schedule_all_pairs(schedule_set)
    schedule_expected = \
        [('AA', 'BB'), ('AA', 'CC'), ('AA', 'DD'),
         ('BB', 'CC'), ('BB', 'DD'), ('CC', 'DD')]
    assert schedule == schedule_expected
    print(f"\nschedule = {schedule}")


def test_schedule_single_pairs():
    schedule_set = ('AA', 'BB', 'CC', 'DD')
    schedule = scoremipsum.schedule.generate_schedule_single_pairs(schedule_set)
    assert len(sorted(schedule)) == 2
    print(f"\nschedule = {schedule}")


def test_schedule_single_pairs_default():
    schedule_set = scoremipsum.data.TEAMS_DEFAULT
    schedule = scoremipsum.schedule.generate_schedule_single_pairs(schedule_set)
    assert len(sorted(schedule)) == 4
    print(f"\ndefault teams schedule = {schedule}")


def test_schedule_single_pairs_mlb_al_east():
    schedule_set = data.TEAMS_MLB_AL_EAST
    schedule = scoremipsum.schedule.generate_schedule_single_pairs(schedule_set)
    assert len(sorted(schedule)) == 2
    for game in schedule:
        for team in game:
            assert team in data.TEAMS_MLB_AL_EAST
    print(f"\nmlb al east schedule = {schedule}")


def test_schedule_single_pairs_nfl_afc_east():
    schedule_set = data.TEAMS_NFL_AFC_EAST
    schedule = scoremipsum.schedule.generate_schedule_single_pairs(schedule_set)
    assert len(sorted(schedule)) == 2
    for game in schedule:
        for team in game:
            assert team in data.TEAMS_NFL_AFC_EAST
    print(f"\nnfl afc east schedule = {schedule}")


def test_schedule_single_pairs_nhl_eastern_atlantic():
    schedule_set = data.TEAMS_NHL_EASTERN_ATLANTIC
    schedule = scoremipsum.schedule.generate_schedule_single_pairs(schedule_set)
    assert len(sorted(schedule)) == 4
    for game in schedule:
        for team in game:
            assert team in data.TEAMS_NHL_EASTERN_ATLANTIC
    print(f"\nnhl eastern atlantic schedule = {schedule}")


if __name__ == '__main__':
    import sys

    sys.exit()



import random


def compute_score_anyball():
    score_list = [0, 1, 2, 3, 5, 8, 13, 21]
    score = (random.choice(score_list) + random.choice(score_list)
             + random.choice(score_list) + random.choice(score_list))
    return score


def compute_score_baseball():
    # casual model based roughly on:  https://gregstoll.com/~gregstoll/baseball/runsperinning.html
    # 73% chance of 0 runs
    # 15% chance of 1 run
    #  7% chance of 2 runs
    #  3% chance of 3 runs
    #  2% chance of 4 runs (or more) -- four max is good enough for now
    # BACKLOG US143: Implement a better weighted range here
    score = 0
    score_list = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 2, 3, 4]
    for inning in range(1, 9):
        score += random.choice(score_list)
    return score


def compute_score_basketball():
    # no model yet, just some random-ish numbers
    score_list = [20, 25, 26, 27, 28, 29, 30, 35]
    score = (random.choice(score_list) + random.choice(score_list)
             + random.choice(score_list) + random.choice(score_list))
    return score


def compute_score_football():
    score_list = [0, 3, 7, 10]
    score = (random.choice(score_list) + random.choice(score_list)
             + random.choice(score_list) + random.choice(score_list))
    return score


def compute_score_hockey():
    score_list = [0, 1, 2]
    score = random.choice(score_list) + random.choice(score_list) + random.choice(score_list)
    return score


def generate_score_anyball(ruleset=None, active_team=None, opposing_team=None):
    """
    return a result_score for Anyball (pseudogame)
    teams are rated 1-5 for Offense / Defense / Special, default 2
    result_score generation:
        actSpecAdj = actSPEC*(0-1)
        oppSpecAdj = oppSPEC*(0-1)
        actScore = actOFF*(0-2) - oppDEF*(0-2) + actSpecAdj, min 0
        oppScore = oppOFF*(0-2) - actDEF*(0-2) + oppSpecAdj, min 0
    :param ruleset:
    :param active_team:
    :param opposing_team:
    :return:
    """
    if ruleset is None:
        pass

    # score = [99, 0]
    score_visitors = compute_score_anyball()
    score_home = compute_score_anyball()

    if score_visitors == score_home:
        score_visitors, score_home = score_adjust_tie(score_visitors, score_home, game="anyball")

    score = [score_visitors, score_home]

    return score


def generate_score_baseball(ruleset=None, active_team=None, opposing_team=None):
    """
    return a result_score for Baseball (US MLB)
    teams are rated 1-5 for Offense / Defense / Pitching, default 2
    result_score generation:
        TBD
    :param ruleset:
    :param active_team:
    :param opposing_team:
    :return:
    """
    if ruleset is None:
        pass

    score_visitors = compute_score_baseball()
    score_home = compute_score_baseball()

    if score_visitors == score_home:
        score_visitors, score_home = score_adjust_tie(score_visitors, score_home, game="baseball")

    score = [score_visitors, score_home]

    return score


def generate_score_basketball(ruleset=None, active_team=None, opposing_team=None):
    """
    return a result_score for Basketball (US NBA)
    teams are rated 1-5 for Offense / Defense / Special, default 2
    result_score generation:
        TBD
    :param ruleset:
    :param active_team:
    :param opposing_team:
    :return:
    """
    if ruleset is None:
        pass

    score_visitors = compute_score_basketball()
    score_home = compute_score_basketball()

    if score_visitors == score_home:
        score_visitors, score_home = score_adjust_tie(score_visitors, score_home, game="basketball")

    score = [score_visitors, score_home]

    return score


def generate_score_football(ruleset=None, active_team=None, opposing_team=None):
    """
    return a result_score for Football (US NFL)
    teams are rated 1-5 for Offense / Defense / Special, default 2
    result_score generation:
        TBD
    :param ruleset:
    :param active_team:
    :param opposing_team:
    :return:
    """
    if ruleset is None:
        pass

    score_visitors = compute_score_football()
    score_home = compute_score_football()

    if score_visitors == score_home:
        score_visitors, score_home = score_adjust_tie(score_visitors, score_home, game="football")

    score = [score_visitors, score_home]

    return score


def generate_score_hockey(ruleset=None, active_team=None, opposing_team=None):
    """
    return a result_score for Hockey (NHL)
    future:  teams are rated 1-5 for Offense / Defense / Special, default 2
    result_score generation:
        hockey (avg 1.0 goals per period, 3 periods) - 0, 1, 2
    :param ruleset:
    :param active_team:
    :param opposing_team:
    :return:
    """
    if ruleset is None:
        pass

    score_visitors = compute_score_hockey()
    score_home = compute_score_hockey()

    if score_visitors == score_home:
        score_visitors, score_home = score_adjust_tie(score_visitors, score_home, game="hockey")

    score = [score_visitors, score_home]

    return score


def score_adjust_tie(score_visitors, score_home, game=None):
    #  print(f"*** score_adjust_for_tie {score_visitors=} {score_home=} ***")
    tiebreak_score_list = []
    if game is None:
        # don't change result_score for unspecified game
        return score_visitors, score_home

    if game == "anyball":
        # don't change result_score for anyball game! #haha
        return score_visitors, score_home

    if game == "baseball":
        tiebreak_score_list = [1, 1, 1, 1, 1, 1, 1, 2, 3, 4]

    if game == "basketball":
        # BACKLOG US144: basketball overtimes need to account for both teams' scores
        tiebreak_score_list = [5, 6, 6, 7, 7, 7, 8, 8, 9]

    if game == "football":
        tiebreak_score_list = [3, 6]

    if game == "hockey":
        # only one option!
        tiebreak_score_list = [1]

    tiebreak_selector = random.randint(0, 1)
    tiebreaker_score = random.choice(tiebreak_score_list)

    if tiebreak_selector:
        score_home += tiebreaker_score
    else:
        score_visitors += tiebreaker_score

    return score_visitors, score_home



#
#   SCOREM
#
"""
game
----------

game functions for the `scoremipsum` module.
"""

import random

from scoremipsum.data import TEAMS_DEFAULT
from scoremipsum.score import generate_score_anyball, generate_score_football, generate_score_hockey
from scoremipsum.util.team import get_team_data


class GameGeneration:
    """
    game generation class for the `scoremipsum` module.
    """

    def __init__(self, teams=None):
        if teams:
            self._teams = teams
        else:
            self._teams = TEAMS_DEFAULT

    def _team(self):
        return random.choice(self._teams)

    # BACKLOG US145:  Fix disconnect for these methods - remove, use as intended and add unit tests, or redesign.
    @staticmethod
    def get_result_anyball(active_team_data=None, opposing_team_data=None):
        """
        :param active_team_data:
        :param opposing_team_data:
        :return:
        """
        ruleset = {'anyball'}

        if not active_team_data:
            active_team_data = get_team_data()
        if not opposing_team_data:
            opposing_team_data = get_team_data()

        score = generate_score_anyball(ruleset, active_team_data, opposing_team_data)
        return score

    @staticmethod
    def get_result_football(active_team_data=None, opposing_team_data=None):
        """
        :param active_team_data:
        :param opposing_team_data:
        :return:
        """
        ruleset = {'football'}

        if not active_team_data:
            active_team_data = get_team_data()
        if not opposing_team_data:
            opposing_team_data = get_team_data()

        score = generate_score_football(ruleset, active_team_data, opposing_team_data)
        return score

    @staticmethod
    def get_result_hockey(active_team_data=None, opposing_team_data=None):
        """
        :param active_team_data:
        :param opposing_team_data:
        :return:
        """
        ruleset = {'hockey'}

        if not active_team_data:
            active_team_data = get_team_data()
        if not opposing_team_data:
            opposing_team_data = get_team_data()

        score = generate_score_hockey(ruleset, active_team_data, opposing_team_data)
        return score
